"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.preparePaginationResolver = preparePaginationResolver;

var _preparePaginationType = require("./types/preparePaginationType");

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const DEFAULT_PER_PAGE = 20;

function preparePaginationResolver(tc, opts) {
  if (!tc || tc.constructor.name !== 'TypeComposer') {
    throw new Error('First arg for prepareConnectionResolver() should be instance of TypeComposer');
  }

  if (!opts.countResolverName) {
    throw new Error(`TypeComposer(${tc.getTypeName()}) provided to composeWithConnection ` + 'should have option `opts.countResolverName`.');
  }

  const countResolver = tc.getResolver(opts.countResolverName);

  if (!countResolver) {
    throw new Error(`TypeComposer(${tc.getTypeName()}) provided to composeWithConnection ` + `should have resolver with name '${opts.countResolverName}' ` + 'due opts.countResolverName.');
  }

  const countResolve = countResolver.getResolve();

  if (!opts.findResolverName) {
    throw new Error(`TypeComposer(${tc.getTypeName()}) provided to composeWithConnection ` + 'should have option `opts.findResolverName`.');
  }

  const findManyResolver = tc.getResolver(opts.findResolverName);

  if (!findManyResolver) {
    throw new Error(`TypeComposer(${tc.getTypeName()}) provided to composeWithConnection ` + `should have resolver with name '${opts.findResolverName}' ` + 'due opts.countResolverName.');
  }

  const findManyResolve = findManyResolver.getResolve();
  const additionalArgs = {};

  if (findManyResolver.hasArg('filter')) {
    const filter = findManyResolver.getArg('filter');

    if (filter) {
      additionalArgs.filter = filter;
    }
  }

  if (findManyResolver.hasArg('sort')) {
    const sort = findManyResolver.getArg('sort');

    if (sort) {
      additionalArgs.sort = sort;
    }
  }

  return new tc.constructor.schemaComposer.Resolver({
    type: (0, _preparePaginationType.preparePaginationTC)(tc),
    name: 'pagination',
    kind: 'query',
    args: _objectSpread({
      page: {
        type: 'Int',
        description: 'Page number for displaying'
      },
      perPage: {
        type: 'Int',
        description: '',
        defaultValue: opts.perPage || DEFAULT_PER_PAGE
      }
    }, additionalArgs),
    // prettier-ignore
    resolve: function () {
      var _resolve = _asyncToGenerator(function* (
      /* :: <TContext> */
      rp
      /* : $Shape<PaginationResolveParams<TContext>> */
      ) {
        let countPromise;
        let findManyPromise;
        const _rp$projection = rp.projection,
              projection = _rp$projection === void 0 ? {} : _rp$projection,
              args = rp.args,
              rawQuery = rp.rawQuery;

        const findManyParams
        /* : $Shape<ResolveParams<any, TContext>> */
        = _objectSpread({}, rp);

        const page = parseInt(args.page, 10) || 1;

        if (page <= 0) {
          throw new Error('Argument `page` should be positive number.');
        }

        const perPage = parseInt(args.perPage, 10) || opts.perPage || DEFAULT_PER_PAGE;

        if (perPage <= 0) {
          throw new Error('Argument `perPage` should be positive number.');
        }

        const countParams
        /* : $Shape<ResolveParams<any, TContext>> */
        = _objectSpread({}, rp, {
          rawQuery,
          args: {
            filter: _objectSpread({}, rp.args.filter)
          }
        });

        if (projection.count || projection.pageInfo && (projection.pageInfo.itemCount || projection.pageInfo.pageCount)) {
          countPromise = countResolve(countParams);
        } else {
          countPromise = Promise.resolve(0);
        }

        if (projection && projection.items) {
          // combine top level projection
          // (maybe somebody add additional fields via rp.projection)
          // and items (record needed fields)
          findManyParams.projection = _objectSpread({}, projection, projection.items);
        } else {
          findManyParams.projection = _objectSpread({}, projection);
        }

        const limit = perPage;
        const skip = (page - 1) * perPage;
        findManyParams.args.limit = limit + 1; // +1 document, to check next page presence

        if (skip > 0) {
          findManyParams.args.skip = skip;
        } // pass findMany ResolveParams to top resolver


        rp.findManyResolveParams = findManyParams;
        rp.countResolveParams = countParams; // This allows to optimize and not actually call the findMany resolver
        // if only the count is projected

        if ((projection.count || projection.pageInfo) && Object.keys(projection).length === 1) {
          findManyPromise = Promise.resolve([]);
        } else {
          findManyPromise = findManyResolve(findManyParams);
        }

        return Promise.all([findManyPromise, countPromise]).then(([items, count]) => {
          const result = {
            count,
            items: items.length > limit ? items.slice(0, limit) : items,
            pageInfo: {
              currentPage: page,
              perPage,
              itemCount: count,
              pageCount: Math.ceil(count / perPage),
              hasPreviousPage: page > 1,
              hasNextPage: items.length > limit || page * perPage < count
            }
          };
          return result;
        });
      });

      return function resolve(_x) {
        return _resolve.apply(this, arguments);
      };
    }()
  });
}